import os
from django.conf import settings
from django.apps import apps
from django.db.migrations.recorder import MigrationRecorder
from dektools.file import normal_path

project_dir = normal_path(settings.BASE_DIR)

project_dir_prefix = project_dir + os.path.sep

env_list = [".venv", "venv", "env"]

project_dir_env_list = [os.path.join(project_dir, env) + os.path.sep for env in env_list]


def list_migration_paths():
    result = {}
    for app_config in apps.get_app_configs():
        app_path = app_config.path
        if app_path.startswith(project_dir_prefix) and all(not app_path.startswith(x) for x in project_dir_env_list):
            migrations_path = os.path.join(app_path, 'migrations')
            if os.path.isdir(migrations_path):
                for item in os.listdir(migrations_path):
                    item_path = os.path.join(migrations_path, item)
                    if os.path.isfile(item_path) and item != '__init__.py' and os.path.splitext(item)[-1] == '.py':
                        result.setdefault(app_config.label, set()).add(item_path)
    return project_dir, result


def list_migration_entries():
    result = {}
    for migration in MigrationRecorder.Migration.objects.all():
        result.setdefault(migration.app, set()).add(migration.name)
    return result



from django.db import models
from psqlextra.models import PostgresPartitionedModel, PostgresModel
from psqlextra.models.partitioned import ModelBase, PostgresPartitionedModelMeta


def get_addition_fields(attrs):
    new_attrs = {}
    for obj_name, obj in attrs.items():
        if isinstance(obj, models.ForeignKey):
            model = obj.remote_field.model
            if issubclass(model, PostgresPartitionedModel):
                field_name = model._partitioning_meta.key[0]
                field_obj = getattr(model, field_name).field
                _, _, ar, kw = field_obj.deconstruct()
                # kw['null'] = True
                new_attrs[f"{obj_name}_{field_name}"] = field_obj.__class__(*ar, **kw)
    return new_attrs


class FkPostgresPartitionedModelMeta(ModelBase):
    def __new__(cls, name, bases, attrs, **kwargs):
        return super().__new__(cls, name, bases, {**attrs, **get_addition_fields(attrs)}, **kwargs)


class FkPostgresPartitionedModel(
    PostgresModel, metaclass=FkPostgresPartitionedModelMeta
):
    class Meta:
        abstract = True
        base_manager_name = "objects"


class FkFullPostgresPartitionedModelMeta(PostgresPartitionedModelMeta):
    def __new__(cls, name, bases, attrs, **kwargs):
        return super().__new__(cls, name, bases, {**attrs, **get_addition_fields(attrs)}, **kwargs)


class FkFullPostgresPartitionedModel(
    PostgresModel, metaclass=FkFullPostgresPartitionedModelMeta
):
    class Meta:
        abstract = True
        base_manager_name = "objects"



from psqlextra.backend.migrations.operations import PostgresAddHashPartition


def psqlextra_partitioning_add_hash(model_name, modulus):
    for remainder in range(modulus):
        yield PostgresAddHashPartition(
            model_name=model_name,
            name=str(remainder),
            modulus=modulus,
            remainder=remainder,
        )
